import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

class FlutterBaseReward{
  String _channelNm;
  MethodChannel _platform;

  var onLoadAdCallback = null;
  var onShowAdCallback = null;
  var onFailAdCallback = null;
  var onCompleteAdCallback = null;
  var onCloseAdCallback = null;
  var onClickAdCallback = null;
  var onSkipAdCallback = null;

  String get channelNm => _channelNm;

  FlutterBaseReward({
    @required String channelName
  }){
    this._channelNm = channelName;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
    await _platform.invokeMethod('makeReward');
  }

  Future<void> load() async {
    await _platform.invokeMethod('load');
  }

  Future<void> show() async {
    await _platform.invokeMethod('show');
  }

  Future<bool> isLoaded() async {
    return await _platform.invokeMethod('isLoaded');
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('setAdInfo', zoneId);
  }

  Future<void> setCallbackListener({
    void Function(String zoneId) onLoadAd,
    void Function(String zoneId) onShowAd,
    void Function(String zoneId) onFailAd,
    void Function(String zoneId) onCompleteAd,
    void Function(String zoneId) onCloseAd,
    void Function(String zoneId) onClickAd,
    void Function(String zoneId) onSkipAd
  }) async {

    if(onLoadAd != null){
      onLoadAdCallback = onLoadAd;
    }
    if(onShowAd != null){
      onShowAdCallback = onShowAd;
    }
    if(onFailAd != null){
      onFailAdCallback = onFailAd;
    }
    if(onCompleteAd != null){
      onCompleteAdCallback = onCompleteAd;
    }
    if(onCloseAd != null){
      onCloseAdCallback = onCloseAd;
    }
    if(onClickAd != null){
      onClickAdCallback = onClickAd;
    }
    if(onSkipAd != null){
      onSkipAdCallback = onSkipAd;
    }
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    String zoneId = call.arguments.toString();
    switch (call.method) {
      case 'onLoadAd':
        if (onLoadAdCallback != null) {
          onLoadAdCallback(zoneId);
        };
        break;
      case 'onShowAd':
        if (onShowAdCallback != null) {
          onShowAdCallback(zoneId);
        };
        break;
      case 'onFailAd':
        if (onFailAdCallback != null) {
          onFailAdCallback(zoneId);
        };
        break;
      case 'onCompleteAd':
        if (onCompleteAdCallback != null) {
          onCompleteAdCallback(zoneId);
        };
        break;
      case 'onCloseAd':
        if (onCloseAdCallback != null) {
          onCloseAdCallback(zoneId);
        };
        break;
      case 'onClickAd':
        if (onClickAdCallback != null) {
          onClickAdCallback(zoneId);
        };
        break;
      case 'onSkipAd':
        if (onSkipAdCallback != null) {
          onSkipAdCallback(zoneId);
        };
        break;
      default:
        throw ('method not defined');
    }
  }
}