import 'package:flutter/foundation.dart' as foundation;

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

class FlutterBidmadCommon {
  static const _DEFAULT_CHANNELNAME = "com.bidmad.sdk.common/Flutter";
  MethodChannel _platform;

  FlutterBidmadCommon(){
    _initChannel(_DEFAULT_CHANNELNAME);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> setDebugging(bool isDebug) async {
    await _platform.invokeMethod('setDebugging', {"isDebug":isDebug});
  }

  Future<String> initBannerChannel() async {
    String chanNm = await _platform.invokeMethod('initBannerChannel');
    return chanNm;
  }

  Future<void> initBannerWidget() async {
    await _platform.invokeMethod('initBannerWidget');
  }

  Future<String> initInterstitialChannel() async {
    String chanNm = await _platform.invokeMethod('initInterstitialChannel');
    return chanNm;
  }

  Future<String> initRewardChannel() async {
    String chanNm = await _platform.invokeMethod('initRewardChannel');
    return chanNm;
  }

  Future<String> initGDPRChannel() async {
    String chanNm = await _platform.invokeMethod('initGDPRforGoogle');
    return chanNm;
  }

  Future<void> setUseArea(bool useArea) async {
    await _platform.invokeMethod('setUseArea', useArea);
  }

  Future<void> setGDPRSetting(bool consent) async {
    await _platform.invokeMethod('setGDPRSetting', consent);
  }

  Future<int> getGDPRSetting() async {
    int consentStatus = await _platform.invokeMethod('getGDPRSetting');
    return consentStatus;
  }

  Future<String> reqAdTrackingAuthorization() async {
    
    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      String attResponse = await _platform.invokeMethod('reqAdTrackingAuthorization');
      return attResponse;
    }
    
    return "Available Only for iOS";

  }

  Future<void> setAdvertiserTrackingEnabled(bool enable) async {
    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      await _platform.invokeMethod('setAdvertiserTrackingEnabled', enable);
    }
  }

  Future<bool> getAdvertiserTrackingEnabled() async {
    if (foundation.defaultTargetPlatform == foundation.TargetPlatform.iOS) {
      bool attEnabled = await _platform.invokeMethod('getAdvertiserTrackingEnabled');
      return attEnabled;
    }   

    return false;
  }



  Future<dynamic> _handleMethod(MethodCall call) async {
    switch (call.method) {
      default:
        throw ('method not defined');
    }
  }
}
