import 'package:flutter/foundation.dart' as foundation;

import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

import 'dart:convert';

class FlutterBidmadGDPRforGoogle{
  String _channelNm;
  MethodChannel _platform;

  var onConsentFormDismissed = null;
  var onConsentFormLoadFailure = null;
  var onConsentFormLoadSuccess = null;
  var onConsentInfoUpdateFailure = null;
  var onConsentInfoUpdateSuccess = null;

  String get channelNm => _channelNm;

  FlutterBidmadGDPRforGoogle({
    @required String channelName
  }){
    this._channelNm = channelName;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
  }

  Future<void> setDebug(String testDeviceId, bool isTestEurope) async {
    await _platform.invokeMethod('setDebug', {'testDeviceId': testDeviceId, 'isTestEurope': isTestEurope});
  }

  Future<void> requestConsentInfoUpdate() async {
    await _platform.invokeMethod('requestConsentInfoUpdate');
  }

  Future<bool> isConsentFormAvailable() async {
    bool consentFormAvailability = await _platform.invokeMethod('isConsentFormAvailable');
    return consentFormAvailability;
  }

  Future<void> loadForm() async {
    await _platform.invokeMethod('loadForm');
  }

  Future<int> getConsentStatus() async {
    int consentStatus = await _platform.invokeMethod('getConsentStatus');
    return consentStatus;
  }

  Future<void> reset() async {
    await _platform.invokeMethod('reset');
  }

  Future<void> showForm() async {
    await _platform.invokeMethod('showForm');
  }

  Future<void> setListener({
    void Function(String errorDescription) onConsentFormDismissed,
    void Function(String errorDescription) onConsentFormLoadFailure,
    void Function() onConsentFormLoadSuccess,
    void Function(String errorDescription) onConsentInfoUpdateFailure,
    void Function() onConsentInfoUpdateSuccess,
  }) async {

    if(onConsentFormDismissed != null){
      this.onConsentFormDismissed = onConsentFormDismissed;
    }
    if(onConsentFormLoadFailure != null){
      this.onConsentFormLoadFailure = onConsentFormLoadFailure;
    }
    if(onConsentFormLoadSuccess != null){
      this.onConsentFormLoadSuccess = onConsentFormLoadSuccess;
    }
    if(onConsentInfoUpdateFailure != null){
      this.onConsentInfoUpdateFailure = onConsentInfoUpdateFailure;
    }
    if(onConsentInfoUpdateSuccess != null){
      this.onConsentInfoUpdateSuccess = onConsentInfoUpdateSuccess;
    }

    _platform.setMethodCallHandler(_handleMethod);
    await _platform.invokeMethod('setListener');
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    String errorDescription;
    if (call.arguments != null) {
      errorDescription = call.arguments.toString();
    }
    switch (call.method) {
      case 'onConsentFormDismissed':
        if (onConsentFormDismissed != null) {
          onConsentFormDismissed(errorDescription);
        };
        break;
      case 'onConsentFormLoadFailure':
        if (onConsentFormLoadFailure != null) {
          onConsentFormLoadFailure(errorDescription);
        };
        break;
      case 'onConsentFormLoadSuccess':
        if (onConsentFormLoadSuccess != null) {
          onConsentFormLoadSuccess();
        };
        break;
      case 'onConsentInfoUpdateFailure':
        if (onConsentInfoUpdateFailure != null) {
          onConsentInfoUpdateFailure(errorDescription);
        };
        break;
      case 'onConsentInfoUpdateSuccess':
        if (onConsentInfoUpdateSuccess != null) {
          onConsentInfoUpdateSuccess();
        };
        break;
      default:
        throw ('method not defined');
    }
  }
}